<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\ActivityBundle\Form\Type;

use Chill\ActivityBundle\Entity\ActivityReasonCategory;
use Chill\MainBundle\Templating\TranslatableStringHelperInterface;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\OptionsResolver\OptionsResolver;
use Symfony\Contracts\Translation\TranslatorInterface;

/**
 * Description of TranslatableActivityReasonCategory.
 */
class TranslatableActivityReasonCategoryType extends AbstractType
{
    private TranslatableStringHelperInterface $translatableStringHelper;

    private TranslatorInterface $translator;

    public function __construct(TranslatableStringHelperInterface $translatableStringHelper, TranslatorInterface $translator)
    {
        $this->translatableStringHelper = $translatableStringHelper;
        $this->translator = $translator;
    }

    public function configureOptions(OptionsResolver $resolver)
    {
        $resolver->setDefaults(
            [
                'class' => ActivityReasonCategory::class,
                'choice_label' => function (ActivityReasonCategory $category) {
                    return $this->translatableStringHelper->localize($category->getName())
                        . (!$category->getActive() ? ' (' . $this->translator->trans('inactive') . ')' : '');
                },
            ]
        );
    }

    public function getBlockPrefix()
    {
        return 'translatable_activity_reason_category';
    }

    public function getParent()
    {
        return EntityType::class;
    }
}
