<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\MainBundle\Repository;

use Chill\MainBundle\Entity\GeographicalUnit;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\ORM\EntityRepository;

class GeographicalUnitRepository implements GeographicalUnitRepositoryInterface
{
    private EntityManagerInterface $em;

    private EntityRepository $repository;

    public function __construct(EntityManagerInterface $em)
    {
        $this->repository = $em->getRepository($this->getClassName());
        $this->em = $em;
    }

    public function find($id): ?GeographicalUnit
    {
        return $this->repository->find($id);
    }

    /**
     * Will return only partial object, where the @see{GeographicalUnit::geom} property is not loaded.
     *
     * @return array|GeographicalUnit[]
     */
    public function findAll(): array
    {
        return $this->repository
            ->createQueryBuilder('gu')
            ->select(sprintf('NEW %s(gu.id, gu.unitName, gu.unitRefId, IDENTITY(gu.layer))', GeographicalUnit\SimpleGeographicalUnitDTO::class))
            ->addOrderBy('IDENTITY(gu.layer)')
            ->addOrderBy(('gu.unitName'))
            ->getQuery()
            ->getResult();
    }

    public function findBy(array $criteria, ?array $orderBy = null, ?int $limit = null, ?int $offset = null): ?GeographicalUnit
    {
        return $this->repository->findBy($criteria, $orderBy, $limit, $offset);
    }

    public function findOneBy(array $criteria): ?GeographicalUnit
    {
        return $this->repository->findOneBy($criteria);
    }

    public function getClassName(): string
    {
        return GeographicalUnit::class;
    }
}
