<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\PersonBundle\Export\Aggregator\AccompanyingCourseAggregators;

use Chill\MainBundle\Export\AggregatorInterface;
use Chill\PersonBundle\Export\Declarations;
use Doctrine\ORM\QueryBuilder;
use LogicException;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Contracts\Translation\TranslatorInterface;

class EmergencyAggregator implements AggregatorInterface
{
    private TranslatorInterface $translator;

    public function __construct(
        TranslatorInterface $translator
    ) {
        $this->translator = $translator;
    }

    public function addRole(): ?string
    {
        return null;
    }

    public function alterQuery(QueryBuilder $qb, $data)
    {
        $qb->addSelect('acp.emergency AS emergency_aggregator');
        $qb->addGroupBy('emergency_aggregator');
    }

    public function applyOn(): string
    {
        return Declarations::ACP_TYPE;
    }

    public function buildForm(FormBuilderInterface $builder)
    {
        // no form
    }

    public function getLabels($key, array $values, $data)
    {
        return function ($value): string {
            if ('_header' === $value) {
                return 'Emergency';
            }

            switch ($value) {
                case true:
                    return $this->translator->trans('is emergency');

                case false:
                    return $this->translator->trans('is not emergency');

                default:
                    throw new LogicException(sprintf('The value %s is not valid', $value));
            }
        };
    }

    public function getQueryKeys($data): array
    {
        return ['emergency_aggregator'];
    }

    public function getTitle(): string
    {
        return 'Group by emergency';
    }
}
